/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "gui_bottom_area.h"
#include "gui_bottom_area_marshallers.h"

#define BOTTOM_WIDTH 164
#define BOTTOM_HIGHT 39

#define BOTTOM_BUTTON_LEFT_NAME_ONLINE "/bottom-left-button-online.png"
#define BOTTOM_BUTTON_LEFT_NAME_OFFLINE "/bottom-left-button-offline.png"
#define BOTTOM_BUTTON_LEFT_NAME_PUSHED "/bottom-left-button-pushed.png"
#define BOTTOM_BUTTON_LEFT_NAME_OVER "/bottom-left-button-over.png"
#define BOTTOM_BUTTON_LEFT_SIZE 32
#define BOTTOM_BUTTON_TOP_OFFSET 3

#define BOTTOM_BUTTON_MIDDLE_NAME "/bottom-middle-button.png"
#define BOTTOM_BUTTON_MIDDLE_NAME_OVER "/bottom-middle-button-over.png"
#define BOTTOM_BUTTON_MIDDLE_NAME_PUSHED "/bottom-middle-button-pushed.png"

#define BOTTOM_BUTTON_RIGHT_NAME "/bottom-right-button.png"
#define BOTTOM_BUTTON_RIGHT_NAME_OVER "/bottom-right-button-over.png"
#define BOTTOM_BUTTON_RIGHT_NAME_PUSHED "/bottom-right-button-pushed.png"
#define BOTTOM_BUTTON_RIGHT_SIZE 30
#define BOTTOM_BUTTON_RIGHT_OFFSET (BOTTOM_BUTTON_RIGHT_SIZE + BOTTOM_BUTTON_TOP_OFFSET)

#define BOTTOM_CORNER_NAME "/bottom-corner.png"
#define BOTTOM_CORNER_SIZE 14

#define GUI_BOTTOM_AREA_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GUI_TYPE_BOTTOM_AREA, GuiBottomAreaPrivate))

G_DEFINE_TYPE (GuiBottomArea, gui_bottom_area, GTK_TYPE_DRAWING_AREA);

static gboolean gui_bottom_area_expose 				(GtkWidget *widget, GdkEventExpose *event);
static gboolean gui_bottom_area_configure_event 	(GtkWidget *widget, GdkEventConfigure *event);
static gboolean gui_bottom_area_button_press 		(GtkWidget *widget, GdkEventButton *event);
static gboolean gui_bottom_area_button_release 		(GtkWidget *widget, GdkEventButton *event);
static gboolean gui_bottom_area_motion_notify 		(GtkWidget *widget, GdkEventMotion *event);
static gboolean gui_bottom_area_leave_notify_event 	(GtkWidget *widget, GdkEventCrossing *event);

static void gui_bottom_area_finalize 	(GuiBottomArea *bottom_area);
static void gui_bottom_area_draw_base	(GtkWidget *widget, cairo_t *cr);
static void gui_bottom_area_draw_corner	(GtkWidget *widget, cairo_t *cr);
static void gui_bottom_area_draw_buttons(GtkWidget *widget, cairo_t *cr);

typedef struct _GuiBottomAreaPrivate GuiBottomAreaPrivate;

struct _GuiBottomAreaPrivate
{
	gboolean hamachi_started;
	gboolean right_button_over;
	gboolean right_button_pushed;

	gboolean middle_button_over;
	gboolean middle_button_pushed;

	gboolean left_button_over;
	gboolean left_button_pushed;
};

enum{
	SIGNAL_STOP_BOTTOM,
	SIGNAL_MIDDLE_BOTTOM,
	SIGNAL_LAST
};

static guint gui_bottom_area_signals[SIGNAL_LAST] = { 0 };

static void
gui_bottom_area_class_init (GuiBottomAreaClass *class)
{
	GObjectClass *obj_class;
	GtkWidgetClass *widget_class;

	obj_class = G_OBJECT_CLASS (class);
	obj_class->finalize = (GObjectFinalizeFunc) gui_bottom_area_finalize;

	widget_class = GTK_WIDGET_CLASS (class);

	gui_bottom_area_signals[SIGNAL_STOP_BOTTOM] = g_signal_new (
					"stop_button_clicked",
					G_OBJECT_CLASS_TYPE (obj_class),
					G_SIGNAL_RUN_FIRST,
					0,
					NULL, NULL,
					_bottom_area_marshal_VOID__INT,
					G_TYPE_NONE, 1,
					G_TYPE_INT);

	gui_bottom_area_signals[SIGNAL_MIDDLE_BOTTOM] = g_signal_new (
					"middle_button_clicked",
					G_OBJECT_CLASS_TYPE (obj_class),
					G_SIGNAL_RUN_FIRST,
					0,
					NULL, NULL,
					_bottom_area_marshal_VOID__INT_INT,
					G_TYPE_NONE, 2,
					G_TYPE_INT,
					G_TYPE_INT);

	widget_class->expose_event = gui_bottom_area_expose;
	widget_class->configure_event = gui_bottom_area_configure_event;
	widget_class->button_press_event = gui_bottom_area_button_press;
	widget_class->button_release_event = gui_bottom_area_button_release;
	widget_class->motion_notify_event = gui_bottom_area_motion_notify;
	widget_class->leave_notify_event = gui_bottom_area_leave_notify_event;

	g_type_class_add_private (obj_class, sizeof (GuiBottomAreaPrivate));
}

static void
gui_bottom_area_init (GuiBottomArea *bottom_area)
{
	GuiBottomAreaPrivate *priv;

	priv = GUI_BOTTOM_AREA_GET_PRIVATE (bottom_area);

	gtk_widget_set_extension_events (GTK_WIDGET (bottom_area), GDK_EXTENSION_EVENTS_CURSOR);

	gtk_widget_add_events (GTK_WIDGET (bottom_area), GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_BUTTON_MOTION_MASK
			 | GDK_POINTER_MOTION_MASK);
}

static void
gui_bottom_area_finalize (GuiBottomArea *top_area)
{

}

static gboolean
gui_bottom_area_configure_event (GtkWidget *widget, GdkEventConfigure *event)
{
	return FALSE;
}

static gboolean
gui_bottom_area_expose (GtkWidget *widget, GdkEventExpose *event)
{
	cairo_t *cr;
    
    cr = gdk_cairo_create(widget->window);	
        
    cairo_rectangle(cr, event->area.x, event->area.y,
		    			event->area.width, event->area.height);
    cairo_clip(cr);	

	gui_bottom_area_draw_base(widget, cr);
	gui_bottom_area_draw_buttons(widget, cr);
	gui_bottom_area_draw_corner(widget, cr);

	cairo_destroy(cr);
	
	return TRUE;
}

static gboolean
gui_bottom_area_button_press (GtkWidget *bottom_area, GdkEventButton *event)
{
	GuiBottomAreaPrivate *priv;

	priv = GUI_BOTTOM_AREA_GET_PRIVATE (GUI_BOTTOM_AREA(bottom_area));

	if(event->button == 1){
		if((event->x > BOTTOM_BUTTON_TOP_OFFSET) &&
			(event->x < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_LEFT_SIZE)) &&
			(event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
			(event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_LEFT_SIZE))){
			if(!priv->left_button_pushed){
				priv->left_button_pushed = TRUE;
				gtk_widget_queue_draw(bottom_area);
			}
			return TRUE;
		}else if((event->x > ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET-BOTTOM_BUTTON_RIGHT_SIZE)) &&
				 (event->x < ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET)) &&
				 (event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
			 	 (event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_RIGHT_SIZE))){
			if(!priv->middle_button_pushed){
				priv->middle_button_pushed = TRUE;
				gtk_widget_queue_draw(bottom_area);
			}		
		}else if((event->x > ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET)) &&
				 (event->x < ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_TOP_OFFSET)) &&
				 (event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
				 (event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_RIGHT_SIZE))){
			if(!priv->right_button_pushed){
				priv->right_button_pushed = TRUE;
				gtk_widget_queue_draw(bottom_area);				
			}		
		}  		
	}
	return FALSE;
}

static gboolean
gui_bottom_area_button_release (GtkWidget *bottom_area, GdkEventButton *event)
{
	gint x, y;
	GuiBottomAreaPrivate *priv;

	priv = GUI_BOTTOM_AREA_GET_PRIVATE (GUI_BOTTOM_AREA(bottom_area));

	if(event->button == 1){
		if((event->x > BOTTOM_BUTTON_TOP_OFFSET) &&
			(event->x < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_LEFT_SIZE)) &&
			(event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
			(event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_LEFT_SIZE))){
			if(priv->hamachi_started){
				priv->hamachi_started = FALSE;
				g_signal_emit (GUI_BOTTOM_AREA(bottom_area),
							gui_bottom_area_signals[SIGNAL_STOP_BOTTOM], 0,
							GUI_BOTTOM_AREA_HAMACHI_STOP);

			}else if(!priv->hamachi_started){
				priv->hamachi_started = TRUE;
				g_signal_emit (GUI_BOTTOM_AREA(bottom_area),
							gui_bottom_area_signals[SIGNAL_STOP_BOTTOM], 0,
							GUI_BOTTOM_AREA_HAMACHI_START);
			}
		}else if((event->x > ((GTK_WIDGET(bottom_area)->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET-BOTTOM_BUTTON_RIGHT_SIZE)) &&
				(event->x < ((GTK_WIDGET(bottom_area)->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET)) &&
				(event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
				(event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_RIGHT_SIZE))){
				x = ((GTK_WIDGET(bottom_area)->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET-BOTTOM_BUTTON_RIGHT_SIZE);
				y = BOTTOM_BUTTON_TOP_OFFSET;
				g_signal_emit (GUI_BOTTOM_AREA(bottom_area),
							gui_bottom_area_signals[SIGNAL_MIDDLE_BOTTOM], 0,
							(gint)x, (gint)y);
		}else if((event->x > ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET)) &&
				 (event->x < ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_TOP_OFFSET)) &&
				 (event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
				 (event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_RIGHT_SIZE))){
				/** TODO: emit signal */				
		}

		if(priv->left_button_pushed){
			priv->left_button_pushed = FALSE;
			gtk_widget_queue_draw(bottom_area);
		}
		if(priv->middle_button_pushed){
			priv->middle_button_pushed = FALSE;
			gtk_widget_queue_draw(bottom_area);
		}
		if(priv->right_button_pushed){
			priv->right_button_pushed = FALSE;
			gtk_widget_queue_draw(bottom_area);
		}
	}

	return FALSE;
}

static gboolean
gui_bottom_area_motion_notify (GtkWidget *bottom_area, GdkEventMotion *event)
{
	GuiBottomAreaPrivate *priv;

	priv = GUI_BOTTOM_AREA_GET_PRIVATE (GUI_BOTTOM_AREA(bottom_area));

	if((event->x > BOTTOM_BUTTON_TOP_OFFSET) &&
		(event->x < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_LEFT_SIZE)) &&
		(event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
		(event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_LEFT_SIZE))){
		if(!priv->left_button_over){
			priv->left_button_over = TRUE;
			gtk_widget_queue_draw(bottom_area);
		}
	}else if(priv->left_button_over){
		priv->left_button_over = FALSE;
		gtk_widget_queue_draw(bottom_area);
	}
	
	if((event->x > ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET-BOTTOM_BUTTON_RIGHT_SIZE)) &&
			(event->x < ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET)) &&
			(event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
			(event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_RIGHT_SIZE))){
		if(!priv->middle_button_over){
			priv->middle_button_over = TRUE;
			gtk_widget_queue_draw(bottom_area);
		}
	}else if(priv->middle_button_over){
		priv->middle_button_over = FALSE;
		gtk_widget_queue_draw(bottom_area);
	}
	
	if((event->x > ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET)) &&
			 (event->x < ((bottom_area->allocation.width-1)-BOTTOM_BUTTON_TOP_OFFSET)) &&
			 (event->y > BOTTOM_BUTTON_TOP_OFFSET) &&
			 (event->y < (BOTTOM_BUTTON_TOP_OFFSET+BOTTOM_BUTTON_RIGHT_SIZE))){
		if(!priv->right_button_over){
			priv->right_button_over = TRUE;
			gtk_widget_queue_draw(bottom_area);
		}
	}else if(priv->right_button_over){
		priv->right_button_over = FALSE;
		gtk_widget_queue_draw(bottom_area);
	}
	return FALSE;
}

static gboolean
gui_bottom_area_leave_notify_event(GtkWidget *bottom_area, GdkEventCrossing *event)
{
	GuiBottomAreaPrivate *priv;

	priv = GUI_BOTTOM_AREA_GET_PRIVATE (GUI_BOTTOM_AREA(bottom_area));

	if(priv->left_button_over){
		priv->left_button_over = FALSE;
		gtk_widget_queue_draw(bottom_area);
	}

	if(priv->middle_button_over){
		priv->middle_button_over = FALSE;
		gtk_widget_queue_draw(bottom_area);
	}

	return FALSE;
}

static void
gui_bottom_area_draw_image(cairo_t *cr, gchar* file, gint x, gint y)
{
	GdkPixbuf* pixb;

	g_return_if_fail(file);
	
	pixb = gdk_pixbuf_new_from_file (file, NULL);

	g_return_if_fail(pixb);
	
	cairo_save(cr);
	gdk_cairo_set_source_pixbuf(cr, pixb, x, y);	
	cairo_paint(cr);
	cairo_restore(cr);  
	g_object_unref (pixb);		
}

static void
gui_bottom_area_draw_corner(GtkWidget *widget, cairo_t *cr)
{
	gchar* file;

	file = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_CORNER_NAME, NULL);
	
	gui_bottom_area_draw_image(cr, file, 
					(widget->allocation.width-1)-BOTTOM_CORNER_SIZE-1, 
					(widget->allocation.height-1)-BOTTOM_CORNER_SIZE-1);
	g_free(file);
}

static void
gui_bottom_area_draw_buttons(GtkWidget *widget, cairo_t *cr)
{
	gchar* right_filename;
	gchar* middle_filename;
	gchar* left_filename;
	GuiBottomAreaPrivate *priv;
	
	priv = GUI_BOTTOM_AREA_GET_PRIVATE (widget);
	
	if(priv->left_button_pushed){
		left_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_LEFT_NAME_PUSHED, NULL);
	}else if(priv->left_button_over){
		left_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_LEFT_NAME_OVER, NULL);
	}else if(priv->hamachi_started){
		left_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_LEFT_NAME_ONLINE, NULL);
	}else{
		left_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_LEFT_NAME_OFFLINE, NULL);
	}
	
	if(priv->middle_button_pushed){
		middle_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_MIDDLE_NAME_PUSHED, NULL);
	}else if(priv->middle_button_over){
		middle_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_MIDDLE_NAME_OVER, NULL);
	}else{
		middle_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_MIDDLE_NAME, NULL);
	}

	if(priv->right_button_pushed){
		right_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_RIGHT_NAME_PUSHED, NULL);
	}else if(priv->right_button_over){
		right_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_RIGHT_NAME_OVER, NULL);
	}else{
		right_filename = g_strconcat(PACKAGE_DATA_DIR, BOTTOM_BUTTON_RIGHT_NAME, NULL);
	}
		
	gui_bottom_area_draw_image(cr, left_filename, 
								BOTTOM_BUTTON_TOP_OFFSET, 
								BOTTOM_BUTTON_TOP_OFFSET);
	gui_bottom_area_draw_image(cr, middle_filename, 
								(widget->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET-BOTTOM_BUTTON_RIGHT_SIZE, 
								BOTTOM_BUTTON_TOP_OFFSET);
	gui_bottom_area_draw_image(cr, right_filename, 
								(widget->allocation.width-1)-BOTTOM_BUTTON_RIGHT_OFFSET, 
								BOTTOM_BUTTON_TOP_OFFSET);								
		
	g_free(right_filename);
	g_free(middle_filename);
	g_free(left_filename);
}

static void
gui_bottom_area_draw_base(GtkWidget *widget, cairo_t *cr)
{
	cairo_save(cr);
	cairo_set_line_width(cr, 1);
	
	cairo_set_source_rgba(cr, 0.35, 0.36, 0.35, 1);	
	cairo_paint(cr);		
	
    /** left frame extern SW -> NW */
    cairo_set_source_rgba(cr, 1, 1, 1, 1);	
  	cairo_move_to(cr, 0.5, widget->allocation.height);
  	cairo_line_to(cr, 0.5, 0); 
 	cairo_stroke(cr); 	 	 	
 
 	/** left frame intern SW -> NW -> NE */
 	cairo_set_source_rgba(cr, 0.56, 0.56, 0.56, 1);	
	cairo_move_to(cr, 1.5, widget->allocation.height);
  	cairo_line_to(cr, 1.5, 0.5);
  	cairo_line_to(cr, widget->allocation.width-2, 0.5); 	
 	cairo_stroke(cr);  	
  
  	/** left black line NE -> SE -> SW*/
  	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 1);	
  	cairo_move_to(cr, widget->allocation.width-0.5, 0);
  	cairo_line_to(cr, widget->allocation.width-0.5, widget->allocation.height-0.5);
  	cairo_line_to(cr, 0, widget->allocation.height-0.5);
  	cairo_stroke(cr);

	/** right frame NE -> SE -> SW */
	cairo_set_source_rgba(cr, 0.184, 0.184, 0.184, 1);
	cairo_move_to(cr, widget->allocation.width-1.5, 0);
	cairo_line_to(cr, widget->allocation.width-1.5, widget->allocation.height-1.5);	
	cairo_line_to(cr, 1, widget->allocation.height-1.5);
	cairo_stroke(cr);
  
	cairo_restore(cr);
}

void
gui_bottom_area_set_hamachi_status(GuiBottomArea *bottom_area, gboolean status)
{
	GuiBottomAreaPrivate *priv;

	priv = GUI_BOTTOM_AREA_GET_PRIVATE(bottom_area);
	priv->hamachi_started = status;
}

GtkWidget *
gui_bottom_area_new (void)
{
	return g_object_new (GUI_TYPE_BOTTOM_AREA, NULL);
}

