/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include "gui_frame.h"
#include "gui_frame_marshallers.h"

#define MARGIN_TOP 20
#define MARGIN_SIDE 10
#define MARGIN_BOTTOM 10

#define RESIZE_AREA_W 10
#define RESIZE_AREA_H 10

#define MOVE_AREA_H 15

#define TOP_BUTTON_SIZE 15
#define TOP_BUTTON_MARGIN 2
#define TOP_BUTTON_TOTAL_SIZE (TOP_BUTTON_SIZE + TOP_BUTTON_MARGIN)

#define TOP_IP_TEXT_MARGIN 15
#define TOP_NICK_TEXT_MARGIN 40

#define TOP_BUTTON_LEFT_OFFSET 2
#define TOP_BUTTON_RIGTH_OFFSET (TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET)

#define TOP_RIGHT_BUTTON_PUSHED "/top-right-button-pushed.png"
#define TOP_RIGHT_BUTTON_OVER "/top-right-button-over.png"
#define TOP_RIGHT_BUTTON "/top-right-button.png"

#define TOP_LEFT_BUTTON_PUSHED "/top-left-button-pushed.png"
#define TOP_LEFT_BUTTON_OVER "/top-left-button-over.png"
#define TOP_LEFT_BUTTON "/top-left-button.png"

#define GUI_FRAME_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GUI_TYPE_FRAME, GuiFramePrivate))

#define COLOR(x) (x*(65535/255))

G_DEFINE_TYPE (GuiFrame, gui_frame, GTK_TYPE_BIN)

static gint gui_frame_expose        (GtkWidget *widget, GdkEventExpose *event);

static gboolean gui_frame_button_press 		(GtkWidget *widget, GdkEventButton *event);
static gboolean gui_frame_button_release 	(GtkWidget *widget, GdkEventButton *event);
static gboolean gui_frame_motion_notify 	(GtkWidget *widget, GdkEventMotion *event);
static gboolean	gui_frame_leave_notify_event(GtkWidget *widget, GdkEventCrossing *event);
static void		gui_frame_realize 			(GtkWidget *widget);
static void 	gui_frame_draw_base			(GtkWidget *widget, cairo_t *cr);
static void 	gui_frame_draw_buttons		(GtkWidget* widget, cairo_t *cr);
static void 	gui_frame_draw_button		(GtkWidget* widget, cairo_t *cr, gchar* path, gint x);
static void 	gui_frame_size_request  	(GtkWidget *widget, GtkRequisition *requisition);
static void 	gui_frame_size_allocate 	(GtkWidget *widget, GtkAllocation  *allocation);
static void 	gui_frame_remove        	(GtkContainer *container, GtkWidget *child);
static void 	gui_frame_forall        	(GtkContainer *container, gboolean include_internals,
			             			 		GtkCallback callback, gpointer callback_data);

static void 	gui_frame_left_button_clicked 	(GuiFrame *frame);
static void 	gui_frame_right_button_clicked 	(GuiFrame *frame);

static GdkPixmap* gui_frame_create_bg_pixmap(GtkWidget *widget);
static GtkStyle*  gui_frame_create_style	(GtkWidget *widget);
static GtkWindow* gui_frame_get_window		(GtkWidget* widget);
typedef struct _GuiFramePrivate GuiFramePrivate;

struct _GuiFramePrivate
{
	GtkAllocation child_allocation;
	
	gboolean left_button_pushed;
	gboolean left_button_over;
	gboolean right_button_over;
	gboolean right_button_pushed;
	
	GdkCursor *curser_corner;
};

enum{
	SIGNAL_LEFT_BUTTON,
	SIGNAL_RIGHT_BUTTON,
	SIGNAL_LAST
};

static guint gui_frame_signals[SIGNAL_LAST] = { 0, 0 };

static void
gui_frame_class_init (GuiFrameClass *class)
{
	GObjectClass *gobject_class;
	GtkWidgetClass *widget_class;
	GtkContainerClass *container_class;

	gobject_class = (GObjectClass*) class;
	widget_class = GTK_WIDGET_CLASS (class);
	container_class = GTK_CONTAINER_CLASS (class);

	gui_frame_signals[SIGNAL_LEFT_BUTTON] = g_signal_new (
					"left-button-clicked",
					G_OBJECT_CLASS_TYPE (gobject_class),
					G_SIGNAL_RUN_FIRST,
					0,NULL, NULL,
					_frame_marshal_VOID__VOID,
					G_TYPE_NONE, 0);

	gui_frame_signals[SIGNAL_RIGHT_BUTTON] = g_signal_new (
					"right-button-clicked",
					G_OBJECT_CLASS_TYPE (gobject_class),
					G_SIGNAL_RUN_FIRST,
					0,NULL, NULL,
					_frame_marshal_VOID__VOID,
					G_TYPE_NONE, 0);

	widget_class->expose_event = gui_frame_expose;
	widget_class->realize = gui_frame_realize;
	widget_class->size_request = gui_frame_size_request;
	widget_class->size_allocate = gui_frame_size_allocate;
	widget_class->button_press_event = gui_frame_button_press;
	widget_class->button_release_event = gui_frame_button_release;
	widget_class->motion_notify_event = gui_frame_motion_notify;
	widget_class->leave_notify_event = gui_frame_leave_notify_event;
	
	container_class->remove = gui_frame_remove;
	container_class->forall = gui_frame_forall;

	g_type_class_add_private (gobject_class, sizeof (GuiFramePrivate));
}

static void
gui_frame_init (GuiFrame *frame)
{
	GuiFramePrivate *priv;

	priv = GUI_FRAME_GET_PRIVATE (frame);
	priv->left_button_pushed 	= FALSE;
	priv->left_button_over		= FALSE;
	priv->right_button_over		= FALSE;
	priv->right_button_pushed	= FALSE;	
	
	GTK_WIDGET_UNSET_FLAGS (frame, GTK_NO_WINDOW);
	
	gtk_widget_set_extension_events (GTK_WIDGET (frame), GDK_EXTENSION_EVENTS_CURSOR);

	gtk_widget_add_events (GTK_WIDGET (frame), GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_BUTTON_MOTION_MASK
			 | GDK_POINTER_MOTION_MASK);
			 
	g_signal_connect (frame, "left-button-clicked",
			G_CALLBACK (gui_frame_left_button_clicked), NULL);

	g_signal_connect (frame, "right-button-clicked",
			G_CALLBACK (gui_frame_right_button_clicked), NULL);			 
}

static void
gui_frame_realize (GtkWidget *widget)
{
	GuiFrame *frame;
	GdkWindowAttr attributes;
	gint attributes_mask;

	g_return_if_fail (GUI_IS_FRAME (widget));

	frame = GUI_FRAME(widget);
	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

	attributes.window_type = GDK_WINDOW_CHILD;
	attributes.x = widget->allocation.x;
	attributes.y = widget->allocation.y;
	attributes.width = widget->allocation.width;
	attributes.height = widget->allocation.height;
	attributes.wclass = GDK_INPUT_OUTPUT;
	attributes.visual = gtk_widget_get_visual (widget);
	attributes.colormap = gtk_widget_get_colormap (widget);
	attributes.event_mask = gtk_widget_get_events (widget) 
			 | GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_BUTTON_MOTION_MASK
			 | GDK_POINTER_MOTION_MASK;

	attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;;

	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget), &attributes, attributes_mask);
	gdk_window_set_user_data (widget->window, frame);

	widget->style = gtk_style_attach (gui_frame_create_style(widget), widget->window);
	gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
}

static void
gui_frame_remove (GtkContainer *container, GtkWidget *child)
{
  GTK_CONTAINER_CLASS (gui_frame_parent_class)->remove (container, child);
}

static void
gui_frame_forall (GtkContainer *container,
		  		  gboolean      include_internals,
		  		  GtkCallback   callback,
		  		  gpointer      callback_data)
{
  GtkBin *bin = GTK_BIN (container);

  if (bin->child)
    (* callback) (bin->child, callback_data);
}

static gboolean
gui_frame_expose (GtkWidget *widget, GdkEventExpose *event)
{
	cairo_t *cr;
	
  	if (GTK_WIDGET_DRAWABLE (widget)){
    	cr = gdk_cairo_create(widget->window);	
        
    	cairo_rectangle(cr, event->area.x, event->area.y,
			    			event->area.width, event->area.height);
    	cairo_clip(cr);	
		gui_frame_draw_base(widget, cr);		
    	gui_frame_draw_buttons(widget, cr);
  		cairo_destroy(cr);
  		
      	(* GTK_WIDGET_CLASS (gui_frame_parent_class)->expose_event) (widget, event);
      	
    }

  	return FALSE;
}

static void
gui_frame_size_request (GtkWidget *widget, GtkRequisition *requisition)
{
  	GtkBin *bin = GTK_BIN (widget);
  	GtkRequisition child_requisition;
 	
 	requisition->width = MARGIN_SIDE * 2;
 	requisition->height = MARGIN_TOP + MARGIN_BOTTOM;
 	 
  	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)){
      gtk_widget_size_request (bin->child, &child_requisition);

      requisition->width  += child_requisition.width;
      requisition->height += child_requisition.height;
	}
}

static void
gui_frame_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
{
  	GtkBin *bin = GTK_BIN (widget);
  	GtkAllocation new_allocation;
  	GuiFramePrivate *priv;
	
	priv = GUI_FRAME_GET_PRIVATE (widget);	
	
  	widget->allocation = *allocation;
  	
  	new_allocation.x = MARGIN_SIDE;
  	new_allocation.y = MARGIN_TOP;
  	new_allocation.width = allocation->width - (MARGIN_SIDE*2);
  	new_allocation.height = allocation->height - MARGIN_TOP - MARGIN_BOTTOM;
  	
  	if(GDK_IS_WINDOW (widget->window))
  		gdk_window_move_resize (widget->window,
								allocation->x,
								allocation->y,
								new_allocation.width + (MARGIN_SIDE*2),
								new_allocation.height + MARGIN_TOP + MARGIN_BOTTOM);
  	
	if (GTK_WIDGET_MAPPED (widget) &&
		(new_allocation.x != priv->child_allocation.x ||
		 new_allocation.y != priv->child_allocation.y ||
		 new_allocation.width != priv->child_allocation.width ||
		 new_allocation.height != priv->child_allocation.height))
		gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
  	
  	if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
    	gtk_widget_size_allocate (bin->child, &new_allocation);  
    	
    priv->child_allocation = new_allocation;
}

static gboolean
gui_frame_button_press (GtkWidget *frame, GdkEventButton *event)
{
	GuiFramePrivate *priv;

	priv = GUI_FRAME_GET_PRIVATE (frame);

	if(event->button == 1){
		if( (event->x > TOP_BUTTON_LEFT_OFFSET) && (event->x < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET) &&
			(event->y > TOP_BUTTON_LEFT_OFFSET) && (event->y < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET)){
			if(!priv->left_button_pushed){
				priv->left_button_pushed = TRUE;
				gtk_widget_queue_draw(frame);
			}
			return TRUE;
		}else if( (event->x > ((frame->allocation.width-1)-TOP_BUTTON_RIGTH_OFFSET)) &&
			(event->x < ((frame->allocation.width-1)-TOP_BUTTON_LEFT_OFFSET)) &&
			(event->y > TOP_BUTTON_LEFT_OFFSET) &&
			(event->y < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET)){
			if(!priv->right_button_pushed){
				priv->right_button_pushed = TRUE;
				gtk_widget_queue_draw(frame);
			}
			return TRUE;
		}else if((event->y > ((frame->allocation.height-1)-RESIZE_AREA_H)) &&
				 (event->x > ((frame->allocation.width-1)-RESIZE_AREA_W))){
			GtkWindow* win = gui_frame_get_window(GTK_WIDGET(frame));
			if(!win) return FALSE;
			gtk_window_begin_resize_drag(win,
										GDK_WINDOW_EDGE_SOUTH_EAST,
										event->button, event->x_root,
										event->y_root, event->time);
			return TRUE;
		}else if(event->y < MOVE_AREA_H){			
			GtkWindow* win = gui_frame_get_window(GTK_WIDGET(frame));
			if(!win) return FALSE;
			gtk_window_begin_move_drag (win,
										event->button, event->x_root,
										event->y_root, event->time);
			return TRUE;
		}
		
	}

	return FALSE;
}

static gboolean
gui_frame_button_release (GtkWidget *frame, GdkEventButton *event)
{
	GuiFramePrivate *priv;

	priv = GUI_FRAME_GET_PRIVATE (frame);

	if(event->button == 1){
		if( (event->x > TOP_BUTTON_LEFT_OFFSET) &&
			(event->x < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET) &&
			(event->y > TOP_BUTTON_LEFT_OFFSET) &&
			(event->y < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET)){
			g_signal_emit (GUI_FRAME(frame),
							gui_frame_signals[SIGNAL_LEFT_BUTTON], 0);
		}else if( (event->x > ((frame->allocation.width-1)-TOP_BUTTON_RIGTH_OFFSET)) &&
			(event->x < ((frame->allocation.width-1)-TOP_BUTTON_LEFT_OFFSET)) &&
			(event->y > TOP_BUTTON_LEFT_OFFSET) &&
			(event->y < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET)){
			g_signal_emit (GUI_FRAME(frame),
							gui_frame_signals[SIGNAL_RIGHT_BUTTON], 0);
		}

		if(priv->right_button_pushed){
			priv->right_button_pushed = FALSE;
			gtk_widget_queue_draw(frame);
		}

		if(priv->left_button_pushed){
			priv->left_button_pushed = FALSE;
			gtk_widget_queue_draw(frame);
		}
	}

	return FALSE;
}

static gboolean
gui_frame_motion_notify (GtkWidget *frame, GdkEventMotion *event)
{
	GuiFramePrivate *priv;

	priv = GUI_FRAME_GET_PRIVATE (frame);

	if( (event->x > TOP_BUTTON_LEFT_OFFSET) && (event->x < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET) &&
		(event->y > TOP_BUTTON_LEFT_OFFSET) && (event->y < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET)){
		if(!priv->left_button_over){
			priv->left_button_over = TRUE;
			gtk_widget_queue_draw(frame);
			return TRUE;
		}
	}else if( (event->x > ((frame->allocation.width-1)-TOP_BUTTON_RIGTH_OFFSET)) &&
			(event->x < ((frame->allocation.width-1)-TOP_BUTTON_LEFT_OFFSET)) &&
			(event->y > TOP_BUTTON_LEFT_OFFSET) &&
			(event->y < TOP_BUTTON_SIZE+TOP_BUTTON_LEFT_OFFSET)){
		if(!priv->right_button_over){
			priv->right_button_over = TRUE;
			gtk_widget_queue_draw(frame);
			return TRUE;
		}
	}else if(priv->left_button_over){
		priv->left_button_over = FALSE;
		gtk_widget_queue_draw(frame);
		return TRUE;
	}else if(priv->right_button_over){
		priv->right_button_over = FALSE;
		gtk_widget_queue_draw(frame);
		return TRUE;
	}else if((event->y > ((frame->allocation.height-1)-RESIZE_AREA_H)) &&
			 (event->x > ((frame->allocation.width-1)-RESIZE_AREA_W))){		
		if(priv->curser_corner == NULL){
			priv->curser_corner = gdk_cursor_new_for_display(gdk_display_get_default(),GDK_BOTTOM_RIGHT_CORNER);
		}
		gdk_window_set_cursor(frame->window, priv->curser_corner);
		return TRUE;
	}else{
		gdk_window_set_cursor(frame->window, NULL);
	}

	return FALSE;
}

static gboolean
gui_frame_leave_notify_event(GtkWidget *top_area, GdkEventCrossing *event)
{
	GuiFramePrivate *priv;

	priv = GUI_FRAME_GET_PRIVATE (top_area);

	if(priv->right_button_over || priv->left_button_over){
		priv->right_button_over = FALSE;
		priv->left_button_over = FALSE;
		gtk_widget_queue_draw(top_area);
	}

	return FALSE;
}

static void
gui_frame_left_button_clicked (GuiFrame *frame)
{
	GtkWindow* win = gui_frame_get_window(GTK_WIDGET(frame));
	g_return_if_fail(win);	
	gtk_window_iconify(win);
}

static void
gui_frame_right_button_clicked(GuiFrame *frame)
{	
	GtkWindow* win = gui_frame_get_window(GTK_WIDGET(frame));
	g_return_if_fail(win);
	gtk_widget_destroy(GTK_WIDGET(win));	
}

static GtkWindow*
gui_frame_get_window(GtkWidget* widget)
{
	while(!GTK_IS_WINDOW(widget)){
		if(!widget) return NULL;		
		widget = gtk_widget_get_parent(widget);
	}	
	return GTK_WINDOW(widget);
}

static void
gui_frame_draw_buttons(GtkWidget* widget, cairo_t *cr)
{	
	GuiFramePrivate *priv;
	gchar* right_filename;
	gchar* left_filename;

	priv = GUI_FRAME_GET_PRIVATE (widget);

	if(priv->right_button_pushed){
		right_filename = g_strconcat(PACKAGE_DATA_DIR, TOP_RIGHT_BUTTON_PUSHED, NULL);
	}else if(priv->right_button_over){
		right_filename = g_strconcat(PACKAGE_DATA_DIR, TOP_RIGHT_BUTTON_OVER, NULL);
	}else{
		right_filename = g_strconcat(PACKAGE_DATA_DIR, TOP_RIGHT_BUTTON, NULL);
	}

	if(priv->left_button_pushed){
		left_filename = g_strconcat(PACKAGE_DATA_DIR, TOP_LEFT_BUTTON_PUSHED, NULL);
	}else if(priv->left_button_over){
		left_filename = g_strconcat(PACKAGE_DATA_DIR, TOP_LEFT_BUTTON_OVER, NULL);
	}else{
		left_filename = g_strconcat(PACKAGE_DATA_DIR, TOP_LEFT_BUTTON, NULL);
	}

	gui_frame_draw_button(widget, cr, right_filename, widget->allocation.width - TOP_BUTTON_TOTAL_SIZE - 1);
	gui_frame_draw_button(widget, cr, left_filename, TOP_BUTTON_MARGIN);

	g_free(right_filename);
	g_free(left_filename);
}

static void
gui_frame_draw_button(GtkWidget* widget, cairo_t *cr, gchar* path, gint x)
{	
	GdkPixbuf* img;

	img = gdk_pixbuf_new_from_file (path, NULL);

	g_return_if_fail(img);
	
	cairo_save(cr);
	gdk_cairo_set_source_pixbuf(cr, img, x, TOP_BUTTON_MARGIN);	
	cairo_paint(cr);
	cairo_restore(cr);  
	g_object_unref (img);	
}

static void
gui_frame_draw_base(GtkWidget *widget, cairo_t *cr)
{
	cairo_save(cr);
	cairo_set_line_width(cr, 1);
	
	cairo_set_source_rgba(cr, 0.37, 0.37, 0.37, 1);	
  	cairo_rectangle(cr, 0.5, 0.5, widget->allocation.width, widget->allocation.height);
	cairo_fill(cr);

    /** left frame extern SW -> NW -> NE */
    cairo_set_source_rgba(cr, 1, 1, 1, 1);	
  	cairo_move_to(cr, 0.5, widget->allocation.height);
  	cairo_line_to(cr, 0.5, 0.5);
  	cairo_line_to(cr, widget->allocation.width, 0.5); 	
 	cairo_stroke(cr);
 
 	/** left frame intern NE -> NW -> SW */
 	cairo_set_source_rgba(cr, 0.56, 0.56, 0.56, 1);	
	cairo_move_to(cr, 1.5, widget->allocation.height);
  	cairo_line_to(cr, 1.5, 1.5);
  	cairo_line_to(cr, widget->allocation.width, 1.5); 	
 	cairo_stroke(cr);
  
  	/** left black line NE -> SE */
  	cairo_set_source_rgba(cr, 0.0, 0.0, 0.0, 1);	
  	cairo_move_to(cr, widget->allocation.width-0.5, 0);
  	cairo_line_to(cr, widget->allocation.width-0.5, widget->allocation.height-0.5);
  	cairo_line_to(cr, 1, widget->allocation.height-0.5);
  	cairo_stroke(cr);

	/** right frame NE -> SE -> SW */
	cairo_set_source_rgba(cr, 0.184, 0.184, 0.184, 1);
	cairo_move_to(cr, widget->allocation.width-1.5, 1);
	cairo_line_to(cr, widget->allocation.width-1.5, widget->allocation.height-1.5);	
	cairo_line_to(cr, 2, widget->allocation.height-1.5);
	cairo_stroke(cr);
  
	cairo_restore(cr);
}

static GdkPixmap*
gui_frame_create_bg_pixmap (GtkWidget *widget)
{	
	gint w, h;
    cairo_t	*cr;
    GdkPixmap *pixmap;
    
    gdk_drawable_get_size(widget->window, &w, &h);
    
    pixmap = gdk_pixmap_new(widget->window, w, h, -1);
    cr = gdk_cairo_create (pixmap);
    	
	cairo_set_source_rgba(cr, 0.37, 0.37, 0.37, 1);	
  	cairo_paint(cr);
    cairo_destroy (cr);
    
    return pixmap;
}

static GtkStyle* 
gui_frame_create_style(GtkWidget *widget)
{
	gint i;
    GtkStyle *style;	    
    GdkPixmap *pixmap;        
    
    style = widget->style;
    
    if(GTK_WIDGET_DRAWABLE(widget))
    	pixmap = gui_frame_create_bg_pixmap (widget);        
    else
    	pixmap = NULL;
        
    for (i = 0; i < 5; i++){
    	if(pixmap)
			style->bg_pixmap[i] = g_object_ref (pixmap);
		
		style->fg[i].red = COLOR(255);
        style->fg[i].blue = COLOR(255);
        style->fg[i].green = COLOR(255);
        style->fg[i].pixel = (gulong)COLOR(255)+COLOR(255)+COLOR(255);
    
		style->bg[i].red = COLOR(141);
        style->bg[i].blue = COLOR(141);
        style->bg[i].green = COLOR(141);
        style->bg[i].pixel = (gulong)COLOR(141)+COLOR(141)+COLOR(141);
    }
    		
	style->fg[GTK_STATE_PRELIGHT].red = COLOR(141);
	style->fg[GTK_STATE_PRELIGHT].blue = COLOR(141);
	style->fg[GTK_STATE_PRELIGHT].green = COLOR(141);
	style->fg[GTK_STATE_PRELIGHT].pixel = (gulong)COLOR(141)+COLOR(141)+COLOR(141);
	       
    return style; 
}

GtkWidget*
gui_frame_new ()
{
  return g_object_new (GUI_TYPE_FRAME, NULL);
}
