/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include "main.h"
#include "callbacks.h"
static volatile gboolean *locks;

#define COLOR(x) (x*(65535/255))

static GtkWidget*
gui_popup_menu_image_item_create(gchar *label, const gchar *stock_id)
{
	GtkWidget *image;
	GdkPixbuf *pixbuf;	
	GtkWidget *item;
		
	item = gtk_image_menu_item_new_with_label(label);
	pixbuf = gtk_widget_render_icon(item, stock_id, GTK_ICON_SIZE_MENU, NULL);	
	image = gtk_image_new_from_pixbuf(pixbuf);
	
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
	
	return item;
} 

static GdkPixmap*
gui_popup_menu_create_bg_pixmap (GdkWindow *win)
{	
	gint w, h;
    cairo_t	*cr;
    GdkPixmap *pixmap;
    
    gdk_drawable_get_size(win, &w, &h);
    
    pixmap = gdk_pixmap_new (win, w, h, -1);
    cr = gdk_cairo_create (pixmap);
    
    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);    
    cairo_rectangle(cr, 0, 0, w, h);
    cairo_set_source_rgba(cr, 0.5, 0.5, 0.5, 1);
    cairo_fill(cr);    
    cairo_destroy (cr);
    
    return pixmap;
}

static void
gui_popup_menu_set_style(GtkWidget *widget)
{
	gint i;
    GtkStyle *style;	    
    GdkPixmap *pixmap;    
    
    style 	= gtk_style_new ();
    pixmap 	= gui_popup_menu_create_bg_pixmap (widget->window);            
    
    for (i = 0; i < 5; i++){		
		style->bg_pixmap[i] = g_object_ref (pixmap);
		
		style->fg[i].red = COLOR(181);
        style->fg[i].blue = COLOR(181);
        style->fg[i].green = COLOR(181);
        style->fg[i].pixel = (gulong)COLOR(181)+COLOR(181)+COLOR(181);
    
		style->bg[i].red = COLOR(141);
        style->bg[i].blue = COLOR(141);
        style->bg[i].green = COLOR(141);
        style->bg[i].pixel = (gulong)COLOR(141)+COLOR(141)+COLOR(141);
    }    
		
	style->fg[GTK_STATE_PRELIGHT].red = COLOR(255);
	style->fg[GTK_STATE_PRELIGHT].blue = COLOR(255);
	style->fg[GTK_STATE_PRELIGHT].green = COLOR(255);
	style->fg[GTK_STATE_PRELIGHT].pixel = (gulong)COLOR(255)+COLOR(255)+COLOR(255);
	       
    gtk_widget_set_style (widget, style);    
}

static gboolean
gui_popup_menu_expose_event_callback (GtkWidget *widget, GdkEventExpose *event, gboolean* run)
{	
	if(!(*run)) return FALSE;
	(*run) = FALSE;
	
	if(GTK_WIDGET_DRAWABLE (widget)){		
		gui_popup_menu_set_style(widget);
		gui_popup_menu_set_style(gtk_widget_get_parent(widget));
				
		if (GTK_BIN (widget)->child){
			GtkWidget *child = GTK_BIN (widget)->child;

			if (GTK_IS_LABEL (child)){		
				gui_popup_menu_set_style (child);
			}
		}
	}
	return FALSE;
}

GtkWidget*
gui_popup_menu_create_client_menu(MainPrivate* priv)
{
	gint i;
	GtkWidget* browse;	
	GtkWidget* ping;
	GtkWidget* vnc;
	GtkWidget* evict;
	GtkWidget* ssh;
	GtkWidget* menu;
	
	locks = g_malloc0(sizeof(gboolean) * 5);
	menu = gtk_menu_new();
	browse = gui_popup_menu_image_item_create("Browse", GTK_STOCK_OPEN);
	ping = gui_popup_menu_image_item_create("Ping", GTK_STOCK_NETWORK);
	vnc = gui_popup_menu_image_item_create("VNC", GTK_STOCK_CONNECT);
	ssh = gui_popup_menu_image_item_create("SSH", GTK_STOCK_CONNECT);
	evict = gui_popup_menu_image_item_create("Evict", GTK_STOCK_CANCEL);
		
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), browse);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), ping);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), vnc);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), ssh);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), evict);
	gtk_widget_show_all(menu);
	
	for(i = 0; i < 5; i++)
		locks[i] = TRUE;
	
	g_signal_connect (G_OBJECT (browse), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[0]);
	g_signal_connect (G_OBJECT (browse), "activate",  
                    G_CALLBACK (popup_menu_client_browse_activate), priv);
                    
	g_signal_connect (G_OBJECT (ping), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[1]);
	g_signal_connect (G_OBJECT (ping), "activate",  
                    G_CALLBACK (popup_menu_client_ping_activate), priv);
                    
	g_signal_connect (G_OBJECT (vnc), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[2]);
	g_signal_connect (G_OBJECT (vnc), "activate",  
                    G_CALLBACK (popup_menu_client_vnc_activate), priv);                    

	g_signal_connect (G_OBJECT (ssh), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[3]);
	g_signal_connect (G_OBJECT (ssh), "activate",  
                    G_CALLBACK (popup_menu_client_ssh_activate), priv);                    
	
	g_signal_connect (G_OBJECT (evict), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[4]);                
	g_signal_connect (G_OBJECT (evict), "activate",  
                    G_CALLBACK (popup_menu_client_evict_activate), priv);                
	
	
	
	return menu;			
}

GtkWidget*
gui_popup_menu_create_network_menu(MainPrivate* priv)
{
	gint i;
	GtkWidget* leave;	
	GtkWidget* delete;
	GtkWidget* status;
	GtkWidget* menu;
	
	locks = g_malloc0(sizeof(gboolean) * 3);
	menu = gtk_menu_new();
	status = gui_popup_menu_image_item_create("Change status", GTK_STOCK_DISCONNECT);
	leave = gui_popup_menu_image_item_create("Leave", GTK_STOCK_QUIT);
	delete = gui_popup_menu_image_item_create("Delete", GTK_STOCK_DELETE);	
		
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), status);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), leave);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), delete);
	gtk_widget_show_all(menu);
	
	for(i = 0; i < 4; i++)
		locks[i] = TRUE;
	
	g_signal_connect (G_OBJECT (status), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[0]);
	g_signal_connect (G_OBJECT (status), "activate",  
                    G_CALLBACK (popup_menu_network_status_activate), priv);      
                                        
	g_signal_connect (G_OBJECT (leave), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[1]);
	g_signal_connect (G_OBJECT (leave), "activate",  
                    G_CALLBACK (popup_menu_network_leave_activate), priv);      
                                                            
	g_signal_connect (G_OBJECT (delete), "expose_event",  
                    G_CALLBACK (gui_popup_menu_expose_event_callback), (gboolean*)&locks[2]);              
	g_signal_connect (G_OBJECT (delete), "activate",  
                    G_CALLBACK (popup_menu_network_delete_activate), priv);      
	return menu;			
}
