/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <glib.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h> /** inet_aton */

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h> /** perror() */

#include "io_ipc.h"

#define IPC_NETWORK_NO_ERROR 1

#define IPC_NETWORK_STATUS_RESP_START -122
#define IPC_NETWORK_STATUS_WRONG_NETWORK 27
#define IPC_NETWORK_STATUS_ALREADY_ONLINE 28
#define IPC_NETWORK_STATUS_ALREADY_OFFLINE 29

#define IPC_NETWORK_DELETE_RESP_START -124
#define IPC_NETWORK_DELETE_NOT_OWNER 19

#define IPC_NETWORK_CREATE_RESP_START -125
#define IPC_NETWORK_CREATE_EXISTS_NOT 18
#define IPC_NETWORK_CREATE_EXISTS 15

#define IPC_NETWORK_JOIN_RESP_START -123
#define IPC_NETWORK_JOIN_WRONG_PASSWD 24
#define IPC_NETWORK_JOIN_NON_EXISTENT_NETWORK 20

#define IPC_NETWORK_EVICT_RESP_START -118

static void ipc_network_create_read_resp		(IPCConnection* connection);
static void	ipc_network_join_read_resp			(IPCConnection* connection);

static gint	ipc_network_status_create_status_msg(gchar* net, gboolean status, gchar* buffer);
static void ipc_network_status_read_resp		(IPCConnection* connection);
static void ipc_network_status_change			(IPCConnection* connection, gchar* net, gint status);

static gint ipc_network_evict_create_msg		(gchar* net, gchar* ip, gchar* buffer);
static void ipc_network_evict_read_response		(IPCConnection* connection);

static gint
ipc_network_status_create_status_msg(gchar* net, gint status, gchar* buffer)
{
	gint size;

	size = strlen(net);

	buffer[0] = IPC_STATUS;
	buffer[1] = size;

	memcpy(&buffer[2], net, size);

	buffer[2 + size] = status;

	return 3 + size;
}

static void
ipc_network_status_read_resp(IPCConnection* connection)
{
	gint read;
	gint size;
	gchar *buff;	
	
	if((read = recv(connection->socket, &size, sizeof(gint), 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}	
	size = ntohl(size);
	buff = g_malloc0(size);
		
	if((read = recv(connection->socket, buff, size, 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}		
	
	if(buff[0] != IPC_NETWORK_STATUS_RESP_START){
		g_debug("Recv unexpected start code.");
		connection->error = IPC_ERROR_PARSE;
		return;
	}
	
	buff += buff[1] + 2;
	
	/** buff[0] is what we did, leave(3)/offline(2)/online(1) */
		
	switch(buff[1]){
		case IPC_NETWORK_NO_ERROR:
			connection->error = IPC_ERROR_NO_ERROR;
		break;
		case IPC_NETWORK_STATUS_ALREADY_OFFLINE:
			connection->error = IPC_ERROR_ALREADY_OFFLINE;
		break;
		case IPC_NETWORK_STATUS_ALREADY_ONLINE:
			connection->error = IPC_ERROR_ALREADY_ONLINE;
		break;		
		case IPC_NETWORK_STATUS_WRONG_NETWORK:
			connection->error = IPC_ERROR_NON_EXISTENT_NETWORK;
		break;
		default:
			connection->error = IPC_ERROR_PARSE;
		break;		
	}	
}

static void
ipc_network_status_change(IPCConnection* connection, gchar* net, gint status)
{
	gint hsize;
	gint nsize;
	gchar buffer[1024];	

	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");

	if((hsize = ipc_network_status_create_status_msg(net, status, (gchar*)&buffer)) == -1){
		connection->error = IPC_ERROR_OTHER;
	}

	nsize = htonl(hsize);

	if(send(connection->socket, &nsize, sizeof(int), 0) == -1){
		perror("send()");
		connection->error = IPC_ERROR_SOCKET;
	}

	if(send(connection->socket, &buffer, hsize, 0) == -1){
		perror("send()");
		connection->error = IPC_ERROR_SOCKET;
	}
	
	connection->error = IPC_ERROR_NO_ERROR;
	
	ipc_network_status_read_resp(connection);
	
	if(pthread_mutex_unlock(&connection->lock) != 0)
		g_printerr("pthread_mutex_unlock error\n");
}

void 
ipc_network_status(IPCConnection* connection, gchar* name, gboolean online)
{
	if(online)
		ipc_network_status_change(connection, name, 1);
	else
		ipc_network_status_change(connection, name, 2);
}

void 
ipc_network_leave(IPCConnection* connection, gchar* name)
{
	ipc_network_status_change(connection, name, 3);
}

static void
ipc_network_create_read_resp(IPCConnection* connection)
{
	gint read;
	gint size;
	gchar *buff;	
	
	if((read = recv(connection->socket, &size, sizeof(gint), 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}	
	size = ntohl(size);
	buff = g_malloc0(size);
		
	if((read = recv(connection->socket, buff, size, 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}		
	
	if(buff[0] != IPC_NETWORK_CREATE_RESP_START){
		g_debug("Recv unexpected start code.");
		connection->error = IPC_ERROR_PARSE;
		return;
	}
		
	switch(buff[1]){
		case IPC_NETWORK_NO_ERROR:
			connection->error = IPC_ERROR_NO_ERROR;
		break;
		case IPC_NETWORK_CREATE_EXISTS:
			connection->error = IPC_ERROR_NETWORK_EXIST;
		break;
		default:
			connection->error = IPC_ERROR_PARSE;
		break;		
	}	
}

void 
ipc_network_create(IPCConnection* connection, gchar* name, gchar* passwd)
{
	gint hsize;
	gint nsize;
	gchar n_size;
	gchar p_size;
	gchar buffer[2048];

	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");

	n_size = (gchar)strlen(name);
	p_size = (gchar)strlen(passwd);

	buffer[0] = (gchar)IPC_CREATE; /** OP code */
	buffer[1] = n_size; /** Size of name */
	
	memcpy(&buffer[2], name, n_size);
	
	buffer[2 + n_size] = p_size; /** Size of passwd */

	memcpy(&buffer[3 + n_size], passwd, buffer[1 + buffer[1]]);
	
	hsize = n_size + p_size + 3; /** total size */
	nsize = htonl(hsize);
	
	/** Send size */
	if(send(connection->socket, &nsize, sizeof(gint), 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(size)");
	}
	
	/** Send Msg */
	if(send(connection->socket, &buffer, hsize, 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(message)");
	}	
	
	ipc_network_create_read_resp(connection);
	
	if(pthread_mutex_unlock(&connection->lock) != 0)
		g_printerr("pthread_mutex_unlock error\n");
}

static void
ipc_network_join_read_resp(IPCConnection* connection)
{	
	gint read;
	gint size;
	gchar *buff;	
	
	if((read = recv(connection->socket, &size, sizeof(gint), 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}	
	size = ntohl(size);
	buff = g_malloc0(size);
		
	if((read = recv(connection->socket, buff, size, 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}		
	
	gint i;
	for(i = 0; i < size; i++){
		g_printerr("%d ", buff[i]);
	}
	g_printerr("\n");
	
	if(buff[0] != IPC_NETWORK_JOIN_RESP_START){
		g_debug("Recv unexpected start code.");
		connection->error = IPC_ERROR_PARSE;
		return;
	}
		
	switch(buff[2]){
		case IPC_NETWORK_NO_ERROR:
			connection->error = IPC_ERROR_NO_ERROR;
		break;
		case IPC_NETWORK_JOIN_WRONG_PASSWD:
			connection->error = IPC_ERROR_WRONG_PASSWD;
		break;
		case IPC_NETWORK_JOIN_NON_EXISTENT_NETWORK:
			connection->error = IPC_ERROR_NON_EXISTENT_NETWORK;
		break;
		default:
			connection->error = IPC_ERROR_PARSE;
		break;		
	}	
}

void 
ipc_network_join(IPCConnection* connection, gchar* name, gchar* passwd)
{
	gint hsize;
	gint nsize;
	gchar n_size;
	gchar p_size;
	gchar buffer[2048];

	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");

	n_size = (gchar)strlen(name);
	p_size = (gchar)strlen(passwd);

	buffer[0] = (gchar)IPC_JOIN; /** OP code */
	buffer[1] = n_size; /** Size of name */
	
	memcpy(&buffer[2], name, n_size);
	
	buffer[2 + n_size] = p_size; /** Size of passwd */

	memcpy(&buffer[3 + n_size], passwd, buffer[1 + buffer[1]]);
	
	hsize = n_size + p_size + 3; /** total size */
	nsize = htonl(hsize);
	
	/** Send size */
	if(send(connection->socket, &nsize, sizeof(gint), 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(size)");
	}
	
	/** Send Msg */
	if(send(connection->socket, &buffer, hsize, 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(message)");
	}	
	
	ipc_network_join_read_resp(connection);
	
	if(pthread_mutex_unlock(&connection->lock) != 0)
		g_printerr("pthread_mutex_unlock error\n");
}

static void
ipc_network_delete_read_resp(IPCConnection* connection)
{
	gint read;
	gint size;
	gchar *buff;	
	
	if((read = recv(connection->socket, &size, sizeof(gint), 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}	
	size = ntohl(size);
	buff = g_malloc0(size);
		
	if((read = recv(connection->socket, buff, size, 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}		
	
	if(buff[0] != IPC_NETWORK_DELETE_RESP_START){
		g_debug("Recv unexpected start code.");
		connection->error = IPC_ERROR_PARSE;
		return;
	}
		
	switch(buff[1]){
		case IPC_NETWORK_DELETE_NOT_OWNER:
			connection->error = IPC_ERROR_NOT_OWNER;
		break;
		case IPC_NETWORK_CREATE_EXISTS_NOT:
			connection->error = IPC_ERROR_NON_EXISTENT_NETWORK;
		break;
		case IPC_NETWORK_NO_ERROR:
			connection->error = IPC_ERROR_NO_ERROR;
		break;
		default:
			connection->error = IPC_ERROR_PARSE;
		break;		
	}
}

void 
ipc_network_delete(IPCConnection* connection, gchar* name)
{
	gint hsize;
	gint nsize;
	gchar name_size;
	gchar buffer[2048];

	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");

	name_size = (gchar)strlen(name);

	buffer[0] = (gchar)IPC_DELETE; /** OP code */
	buffer[1] = name_size; /** Size of name */
	
	memcpy(&buffer[2], name, name_size);	
	
	hsize = name_size + 2; /** total size */
	nsize = htonl(hsize);
	
	/** Send size */
	if(send(connection->socket, &nsize, sizeof(gint), 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(size)");
	}
	
	/** Send Msg */
	if(send(connection->socket, &buffer, hsize, 0) == -1){
		connection->error = IPC_ERROR_SOCKET;
		perror("send(message)");
	}	
	
	ipc_network_delete_read_resp(connection);
	
	if(pthread_mutex_unlock(&connection->lock) != 0)
		g_printerr("pthread_mutex_unlock error\n");
}

static gint 
ipc_network_evict_create_msg(gchar* net, gchar* ip, gchar* buffer)
{
	gint err;
	gint siz;
	struct in_addr addr;

	siz = strlen(net);
	err = inet_aton(ip, (struct in_addr*)&addr);

	if(err == -1)
		return -1;

	buffer[0] = IPC_EVICT;
	buffer[1] = siz;

	memcpy(&buffer[2], net, siz);

	buffer[2 + siz] = '\6';
	buffer[3 + siz] = '\7';
	buffer[4 + siz] = '\1';

	memcpy(&buffer[5 + siz], &addr, sizeof(struct in_addr));

	return 5 + siz + sizeof(struct in_addr);
}

static void
ipc_network_evict_read_response(IPCConnection* connection)
{
	gint read;
	gint size;
	gchar buff[1024];
	
	if((read = recv(connection->socket, &size, sizeof(gint), 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}	
	
	size = ntohl(size);
	
	if(size > 1024){
		connection->error = IPC_ERROR_OTHER;
		return;
	}
	
	if((read = recv(connection->socket, &buff, size, 0)) == -1){
		connection->error = IPC_ERROR_SOCKET;	
		return;
	}	
	
	if(buff[0] != IPC_NETWORK_EVICT_RESP_START){
		g_debug("Recv unexpected start code.");
		connection->error = IPC_ERROR_PARSE;
		return;
	}
	
	if(buff[1] != 1){
		g_debug("Got strange resp code...");
	}
		
	connection->error = IPC_ERROR_NO_ERROR;
}

void
ipc_network_evict(IPCConnection* connection, gchar* net, gchar* ip)
{
	gint hsize;
	gint nsize;
	gchar buffer[1024];

	if(pthread_mutex_lock(&connection->lock) != 0)
		g_printerr("Error locking mutex\n");

	if((hsize = ipc_network_evict_create_msg(net, ip, (gchar*)&buffer)) == -1){
		connection->error = IPC_ERROR_OTHER;
	}

	nsize = htonl(hsize);

	if(send(connection->socket, &nsize, sizeof(gint), 0) == -1){
	    connection->error = IPC_ERROR_CONNECT;
	    perror("send(size)");
	}

	if(send(connection->socket, &buffer, hsize, 0) == -1){
		connection->error = IPC_ERROR_CONNECT;
		perror("send(message)");
	}

	ipc_network_evict_read_response(connection);
	
	if(connection->error != IPC_ERROR_NO_ERROR)
		return;

	connection->error = IPC_ERROR_NO_ERROR;
	
	if(pthread_mutex_unlock(&connection->lock) != 0)
		g_printerr("pthread_mutex_unlock error\n");
}
